# BLE通信でアドバータイジングパケット(Adevertising Data)を生成する為のヘルパー関数
# アドバータイジング PDU = [Header][Adevertiser's address] [Length][AD Type][AD Data]
# 2021/5/19 きむ茶工房(きむしげ)  ブロードキャスト用のアドバータイジングパケットを追加

from micropython import const
import struct
import bluetooth

# アドバータイジングパケット(Adevertising Data)は、次の形式の繰り返しパケットです。
# [Length][AD Type][AD Data]は31byte以内なら繰り返し設定可能です。
# 1バイトのデータ長(N + 1) : [Length]
# 1バイト型(以下の定数を参照:データの種別を表す) : [AD Type]
# Nバイトのタイプ固有のデータ : [AD Data](29byte以内)->[FLAGs(8bit)][Data(28byte)]
# free(27Byte)の場合は[Length][AD Type(0xFF)][AD Data:(0xFFFF)(char free)]

_ADV_TYPE_FLAGS = const(0x01)            # GAP通信における制御情報
_ADV_TYPE_NAME = const(0x09)             # デバイスの名称(完全版)(28Bを超えるなら0x08の短縮名称を使う)
_ADV_TYPE_UUID16_COMPLETE = const(0x3)   # 利用可能な16ビットUUID(全てのUUIDをAD Dataに含める場合)
_ADV_TYPE_UUID32_COMPLETE = const(0x5)
_ADV_TYPE_UUID128_COMPLETE = const(0x7)
_ADV_TYPE_UUID16_MORE = const(0x2)       # 利用可能な16ビットUUID(一部のUUIDのみAD Dataに含める場合)
_ADV_TYPE_UUID32_MORE = const(0x4)
_ADV_TYPE_UUID128_MORE = const(0x6)
_ADV_TYPE_APPEARANCE = const(0x19)       # デバイスの機能種別名
_ADV_TYPE_SPECIFIC_DATA = const(0xFF)    # 任意の送信データ(ベンダー固有のデータ)を送れる
_ADV_DATA_COMPANY_ID = const(0xFFFF)     # ベンダーのID

# gap_advertise(adv_data = ...)に渡されるメッセージパケットを生成します。
# limited_disc : False=LE 一般検出可能モード　True=LE 限定検出可能モード
# br_edr : False=BR/EDRはサポートされていません　True=0x18?
# name : デバイス名で任意につけてOK、iphone等のアプリ画面上にこの装置名が表示されます
# services : サービスのUUID
# appearance : https://www.bluetooth.com/wp-content/uploads/Sitecore-Media-Library/Gatt/Xml/Characteristics/org.bluetooth.characteristic.gap.appearance.xml
def advertising_payload(limited_disc=False, br_edr=False, name=None, services=None, appearance=0, free=None):
    payload = bytearray()

    # アドバータイジングパケットの１データをパックします。
    def _append(adv_type, value):
        nonlocal payload
        payload += struct.pack("BB", len(value) + 1, adv_type) + value

    if free:
        # アドバータイジングで通知するフリーデータをパックします。
        _l = len(free)
        payload += struct.pack("BB", _l + 3, _ADV_TYPE_SPECIFIC_DATA)
        _f = 'H' + str(_l) + 's'
        payload += struct.pack(_f, _ADV_DATA_COMPANY_ID, free)
    else:
        # FLAGsデータをパックします。
        _append(
            _ADV_TYPE_FLAGS,
            struct.pack("B", (0x01 if limited_disc else 0x02) + (0x18 if br_edr else 0x04)),
        )

    if name:
        _append(_ADV_TYPE_NAME, name)

    if services:
        for uuid in services:
            b = bytes(uuid)
            if len(b) == 2:
                _append(_ADV_TYPE_UUID16_COMPLETE, b)
            elif len(b) == 4:
                _append(_ADV_TYPE_UUID32_COMPLETE, b)
            elif len(b) == 16:
                _append(_ADV_TYPE_UUID128_COMPLETE, b)

    # org.bluetooth.characteristic.gap.appearance.xmlを参照してください。（デバイスの機能種別名）
    if appearance:
        _append(_ADV_TYPE_APPEARANCE, struct.pack("<h", appearance))

    return payload

# 以下のコードは作成したアドバータイジングパケットデータの内容確認用関数です。
# セントラル側でのデコードにも使われている様です

def decode_field(payload, adv_type):
    i = 0
    result = []
    while i + 1 < len(payload):
        if payload[i + 1] == adv_type:
            result.append(payload[i + 2 : i + payload[i] + 1])
        i += 1 + payload[i]
    return result

def decode_name(payload):
    n = decode_field(payload, _ADV_TYPE_NAME)
    return str(n[0], "utf-8") if n else ""

def decode_services(payload):
    services = []
    for u in decode_field(payload, _ADV_TYPE_UUID16_COMPLETE):
        services.append(bluetooth.UUID(struct.unpack("<h", u)[0]))
    for u in decode_field(payload, _ADV_TYPE_UUID32_COMPLETE):
        services.append(bluetooth.UUID(struct.unpack("<d", u)[0]))
    for u in decode_field(payload, _ADV_TYPE_UUID128_COMPLETE):
        services.append(bluetooth.UUID(u))
    return services

# 使い方のサンプルデモ
def demo():
    payload = advertising_payload(
        name="micropython",
        services=[bluetooth.UUID(0x181A), bluetooth.UUID("6E400001-B5A3-F393-E0A9-E50E24DCCA9E")],
    )
    print(payload)
    print(decode_name(payload))
    print(decode_services(payload))

if __name__ == "__main__":
    demo()
