# このサンプルでは、BLE機器の【電池の残量を確認する為のサービス】を見つけて接続します。
# 3秒毎にキャラクタリスティックのデータを読み取り(READ)します。
# ペリフェラルから通知を受けたら表示を行います。

import bluetooth
import struct
import time
from ble_advertising import decode_name

from micropython import const

_IRQ_CENTRAL_CONNECT = const(1)
_IRQ_CENTRAL_DISCONNECT = const(2)
_IRQ_GATTS_WRITE = const(3)
_IRQ_GATTS_READ_REQUEST = const(4)
_IRQ_SCAN_RESULT = const(5)
_IRQ_SCAN_DONE = const(6)
_IRQ_PERIPHERAL_CONNECT = const(7)
_IRQ_PERIPHERAL_DISCONNECT = const(8)
_IRQ_GATTC_SERVICE_RESULT = const(9)
_IRQ_GATTC_SERVICE_DONE = const(10)
_IRQ_GATTC_CHARACTERISTIC_RESULT = const(11)
_IRQ_GATTC_CHARACTERISTIC_DONE = const(12)
_IRQ_GATTC_DESCRIPTOR_RESULT = const(13)
_IRQ_GATTC_DESCRIPTOR_DONE = const(14)
_IRQ_GATTC_READ_RESULT = const(15)
_IRQ_GATTC_READ_DONE = const(16)
_IRQ_GATTC_WRITE_DONE = const(17)
_IRQ_GATTC_NOTIFY = const(18)
_IRQ_GATTC_INDICATE = const(19)

_ADV_IND = const(0x00)
_ADV_DIRECT_IND = const(0x01)
_ADV_SCAN_IND = const(0x02)
_ADV_NONCONN_IND = const(0x03)

# バッテリサービス
_BATTERY_DEVICE_NAME = b'RNF0A9'               # 各々のデバイス名により書き換えます
_BATTERY_SERVICE_UUID = bluetooth.UUID(0x180F)
# バッテリレベル特性のキャラクタリスティック
_BATTERY_CHAR_UUID = bluetooth.UUID(0x2A19)


class BLEBatteryCentral:
    def __init__(self, ble):
        self._ble = ble
        self._ble.active(True)     # bluetoothをアクティブにする
        self._ble.irq(self._irq)   # イベントのコールバック関数を登録する

        self._reset()              # 使用する変数の初期化

    def _reset(self):
        # スキャンが成功した時にキャッシュされる名前とアドレスの初期化。
        self._name = None
        self._addr_type = None
        self._addr = None

        # 接続されたデバイス。
        self._conn_handle = None
        self._start_handle = None
        self._end_handle = None
        self._value_handle = None

        # キャッシュされた値(在る場合)
        self._value = None

        # さまざまな操作を完了する為のコールバック。
        # これらは、呼び出された後、Noneにリセットされます。
        self._scan_callback = None
        self._conn_callback = None    # このサンプルはnoneのまま未使用
        self._read_callback = None

        # 新しいデータがペリフェラルから通知された時の永続的なコールバック。
        self._notify_callback = None  # このサンプルはnoneのまま未使用

    def _irq(self, event, data):
        if event == _IRQ_SCAN_RESULT:
            # シングルスキャン結果
            addr_type, addr, adv_type, rssi, adv_data = data
            _name = ''
            for i in _BATTERY_DEVICE_NAME:
                _name += struct.pack("B", i).decode()
            if adv_type in (_ADV_IND, _ADV_DIRECT_IND) and _name in decode_name(adv_data):
                print('scan ok')
                # 潜在的なペリフェラルを見つけ、それを覚えてスキャンを停止します。
                self._addr_type = addr_type
                # addrバッファーは呼び出し元が所有している為、コピーする必要があります。
                self._addr = bytes(addr)
                self._name = _name or "?"
                self._ble.gap_scan(None)  # スキャンを停止

        elif event == _IRQ_SCAN_DONE:
            # スキャン期間が終了したか、手動で停止しました。
            if self._scan_callback:
                if self._addr:
                    # スキャン中にデバイスが見つかりました(スキャンが明示的に停止されました)。
                    self._scan_callback(self._addr_type, self._addr, self._name)
                    self._scan_callback = None
                else:
                    # スキャンがタイムアウトしました。
                    self._scan_callback(None, None, None)

        elif event == _IRQ_PERIPHERAL_CONNECT:
            # ペリフェラル(サーバー)に接続成功しました。
            conn_handle, addr_type, addr = data
            if addr_type == self._addr_type and addr == self._addr:
                self._conn_handle = conn_handle
                # 接続したサーバーに、サービスについて問い合わせます。
                self._ble.gattc_discover_services(self._conn_handle)

        elif event == _IRQ_PERIPHERAL_DISCONNECT:
            # 接続されているペリフェラルが切断されました。
            conn_handle, _, _ = data
            if conn_handle == self._conn_handle:
                self._reset()  # 使用する変数の初期化

        elif event == _IRQ_GATTC_SERVICE_RESULT:
            # 接続されたサーバーがサービスを返しました。
            conn_handle, start_handle, end_handle, uuid = data
            if conn_handle == self._conn_handle and uuid == _BATTERY_SERVICE_UUID:
                self._start_handle, self._end_handle = start_handle, end_handle

        elif event == _IRQ_GATTC_SERVICE_DONE:
            # サービス検出が完了しました。
            print('_IRQ_GATTC_SERVICE_DONE')  # debug
            if self._start_handle and self._end_handle:
                # 接続したサーバーに、指定した範囲のキャラクタリスティックについて問い合わせます。
                self._ble.gattc_discover_characteristics(
                    self._conn_handle, self._start_handle, self._end_handle
                )
            else:
                # バッテリサービスが見つかりませんでした。
                print("Failed to find Battery service.")

        elif event == _IRQ_GATTC_CHARACTERISTIC_RESULT:
            # 接続されたサーバーがバッテリレベル特性(キャラクタリスティック)を返しました。
            conn_handle, def_handle, value_handle, properties, uuid = data
            if conn_handle == self._conn_handle and uuid == _BATTERY_CHAR_UUID:
                self._value_handle = value_handle

        elif event == _IRQ_GATTC_CHARACTERISTIC_DONE:
            # キャラクタリスティック検出が完了しました。
            print('_IRQ_GATTC_CHARACTERISTIC_DONE')  # debug
            if self._value_handle:
                # サーバーの接続と検出が完了しました。接続コールバックを起動します(現在未使用)。
                if self._conn_callback:
                    self._conn_callback()
            else:
                # バッテリレベル特性(キャラクタリスティック)が見つかりませんでした。
                print("Failed to find Battery Level characteristic.")

        elif event == _IRQ_GATTC_READ_RESULT:
            # gattc_read()操作が完了しました。
            conn_handle, value_handle, char_data = data
            if conn_handle == self._conn_handle and value_handle == self._value_handle:
                self._update_value(char_data)  # バッテリ値の取り出し
                if self._read_callback:
                    self._read_callback(self._value)  # print表示
                    self._read_callback = None

        elif event == _IRQ_GATTC_READ_DONE:
            # 読み取りが完了しました（操作なし）。
            conn_handle, value_handle, status = data

        elif event == _IRQ_GATTC_NOTIFY:
            # サーバーが通知要求を送信しました.
            print('_IRQ_GATTC_NOTIFY')  # debug
            conn_handle, value_handle, notify_data = data
            if conn_handle == self._conn_handle and value_handle == self._value_handle:
                self._update_value(notify_data)  # バッテリ値の取り出し
                if self._notify_callback:
                    self._notify_callback(self._value)

    # 接続のチェックを行います、接続中の場合はtrueを返します。
    def is_connected(self):
        return self._conn_handle is not None and self._value_handle is not None

    # スキャンしてバッテリサービスをadvertisingするデバイスを見つけます。
    def scan(self, callback=None):
        self._addr_type = None
        self._addr = None
        self._scan_callback = callback
        # 3秒の間に50ms間隔で50ms期間スキャンを実行します。
        self._ble.gap_scan(3000, 50000, 50000)

    # 指定されたデバイスに接続します(それ以外の場合は、スキャンでキャッシュされたアドレスを使用します)。
    def connect(self, addr_type=None, addr=None, callback=None):
        self._addr_type = addr_type or self._addr_type
        self._addr = addr or self._addr
        self._conn_callback = callback
        if self._addr_type is None or self._addr is None:
            return False
        self._ble.gap_connect(self._addr_type, self._addr)  # ペリフェラルに接続します。
        return True

    # 現在のデバイスから切断します。(未使用)
    def disconnect(self):
        if not self._conn_handle:
            return
        self._ble.gap_disconnect(self._conn_handle)
        self._reset()  # 使用する変数の初期化

    # (非同期)読み取りを発行します(readコールバックのセット)。
    def read(self, callback):
        if not self.is_connected():
            return
        self._read_callback = callback
        self._ble.gattc_read(self._conn_handle, self._value_handle)

    # デバイスが通知した時に呼び出されるコールバックを設定します。(未使用)
    def on_notify(self, callback):
        self._notify_callback = callback

    def _update_value(self, data):
        # データは1バイトで16進数(00-FF)で来る、有効値は00(0)%　- 64(100)%です。
        self._value = str(data[0]) + "%"
        return self._value


def demo():
    ble = bluetooth.BLE()
    # ブルートゥースの有効
    central = BLEBatteryCentral(ble)

    not_found = False

    # スキャン結果のコールバック関数
    def on_scan(addr_type, addr, name):
        if addr_type is not None:
            _rcv_adrs = ''
            for i in range(0, len(addr)):
                _rcv_adrs += hex(addr[i])[2:] + ':'
            print("Scan result found the peripheral:", addr_type, _rcv_adrs[:-1], name)
            central.connect()       # 指定されたデバイスに接続します
        else:
            nonlocal not_found      # デバイスが無い
            not_found = True
            print("No peripheral found.")

    central.scan(callback=on_scan)  # スキャンの実行

    # 接続を待つ...
    while not central.is_connected():
        time.sleep_ms(100)
        if not_found:               # デバイスが無いなら終了
            return

    print("Connected")

    # 明示的に読み取りを発行します(コールバックとして「print」を使用)。
    while central.is_connected():
        central.read(callback=print)
        time.sleep_ms(3000)  # 3秒毎に読み取り

    print("Disconnected")


if __name__ == "__main__":
    demo()
