# このサンプルでは、オブザーバーとして動作しブロードキャスタからのアドバタイズを探します。
# 探せたらアドバタイズのメッセージ表示を行います。

import bluetooth
import time
from ble_advertising import decode_name

from micropython import const

_IRQ_SCAN_RESULT = const(5)
_IRQ_SCAN_DONE = const(6)

_ADV_IND = const(0x00)
_ADV_DIRECT_IND = const(0x01)
_ADV_SCAN_IND = const(0x02)
_ADV_NONCONN_IND = const(0x03)

# アドバタイザーのデバイスアドレス（各々のデバイスアドレスにより書き換えます）
_ADVERTISER_DEVICE_ADDR = '68:27:19:0A:F0:A9'


class BLEObserverCentral:
    def __init__(self, ble):
        self._ble = ble
        self._ble.active(True)     # bluetoothをアクティブにする
        self._ble.irq(self._irq)   # イベントのコールバック関数を登録する

        self._reset()              # 使用する変数の初期化

    def _reset(self):
        # スキャンが成功した時にキャッシュされる名前とアドレスの初期化。
        self._name = None
        self._addr_type = None
        self._addr = None

        # さまざまな操作を完了する為のコールバック。
        # これらは、呼び出された後、Noneにリセットされます。
        self._scan_callback = None

    def _irq(self, event, data):
        if event == _IRQ_SCAN_RESULT:
            # シングルスキャン結果
            addr_type, addr, adv_type, rssi, adv_data = data
            # 受信したアドレス
            _rcv_adrs = ''
            for i in range(0, len(addr)):
                _rcv_adrs += str(addr[i])
            # アドバタイザーのデバイスアドレス
            _adv_adrs = ''
            _d = _ADVERTISER_DEVICE_ADDR.split(":")
            for i in _d:
                _adv_adrs += str(int(i, 16))
            if _rcv_adrs in _adv_adrs:
                # アドレスが一致したら処理する
                print('scan ok')
                import binascii
                # アドバタイズのメッセージを取り出す
                _h = ''
                _d = ''
                _i = ''
                for i in range(0, len(adv_data)):
                    if i == 4 or i == 5:
                        _i += hex(adv_data[i])[2:]
                    if i > 6:
                        _d += str(adv_data[i])
                        _h += hex(adv_data[i])[2:]
                _h = _h.encode()
                print('ベンダーID:{0}'.format(_i))
                print('整数:{0} HEX:{1} 文字:{2}'.format(_d, _h, binascii.unhexlify(_h)))
                # 潜在的なペリフェラルを見つけ、それを覚えてスキャンを停止します。
                self._addr_type = addr_type
                # addrバッファーは呼び出し元が所有している為、コピーする必要があります。
                self._addr = bytes(addr)
                self._name = decode_name(adv_data) or "???"
                self._ble.gap_scan(None)  # スキャンを停止

        elif event == _IRQ_SCAN_DONE:
            # スキャン期間が終了したか、手動で停止しました。
            if self._scan_callback:
                if self._addr:
                    # スキャン中にデバイスが見つかりました(スキャンが明示的に停止されました)。
                    self._scan_callback(self._addr_type, self._addr, self._name)
                    self._scan_callback = None
                else:
                    # スキャンがタイムアウトしました。
                    self._scan_callback(None, None, None)

    # スキャンしてブロードキャスタがadvertisingするデバイスを見つけます。
    def scan(self, callback=None):
        self._addr_type = None
        self._addr = None
        self._name = None
        self._scan_callback = callback
        # 3秒の間に50ms間隔で50ms期間スキャンを実行します。
        self._ble.gap_scan(3000, 50000, 50000)


def demo():
    ble = bluetooth.BLE()
    # ブルートゥースの有効
    central = BLEObserverCentral(ble)

    # スキャン結果のコールバック関数
    def on_scan(addr_type, addr, name):
        if addr_type is not None:
            _rcv_adrs = ''
            for i in range(0, len(addr)):
                _rcv_adrs += hex(addr[i])[2:] + ':'
            print("Scan result found the peripheral:", addr_type, _rcv_adrs[:-1], name)
        else:
            # デバイスが無い
            print("No peripheral found.")

    central.scan(callback=on_scan)  # スキャンの実行(１回だけのスキャン)

    while True:
        time.sleep(1000)


if __name__ == "__main__":
    demo()
