# このサンプルでは、BLE機器の【UART通信サービス】を見つけて接続します。
# 3秒毎にキャラクタリスティックのデータを読み取り(READ)します。
# ペリフェラルから通知を受けたら表示を行います。

import bluetooth
import struct
import time
from ble_advertising import decode_name

from micropython import const

_IRQ_CENTRAL_CONNECT = const(1)
_IRQ_CENTRAL_DISCONNECT = const(2)
_IRQ_GATTS_WRITE = const(3)
_IRQ_GATTS_READ_REQUEST = const(4)
_IRQ_SCAN_RESULT = const(5)
_IRQ_SCAN_DONE = const(6)
_IRQ_PERIPHERAL_CONNECT = const(7)
_IRQ_PERIPHERAL_DISCONNECT = const(8)
_IRQ_GATTC_SERVICE_RESULT = const(9)
_IRQ_GATTC_SERVICE_DONE = const(10)
_IRQ_GATTC_CHARACTERISTIC_RESULT = const(11)
_IRQ_GATTC_CHARACTERISTIC_DONE = const(12)
_IRQ_GATTC_DESCRIPTOR_RESULT = const(13)
_IRQ_GATTC_DESCRIPTOR_DONE = const(14)
_IRQ_GATTC_READ_RESULT = const(15)
_IRQ_GATTC_READ_DONE = const(16)
_IRQ_GATTC_WRITE_DONE = const(17)
_IRQ_GATTC_NOTIFY = const(18)
_IRQ_GATTC_INDICATE = const(19)

_ADV_IND = const(0x00)
_ADV_DIRECT_IND = const(0x01)
_ADV_SCAN_IND = const(0x02)
_ADV_NONCONN_IND = const(0x03)

# UART通信サービス
_UART_DEVICE_NAME = b'UARTF0A9'  # 各々のデバイス名により書き換えます
_UART_SERVICE_UUID = bluetooth.UUID("4e234e52-5309-435b-a3fe-e6da3c1379ee")
# UART通信特性のキャラクタリスティック
_UART_TX_CHAR_UUID = bluetooth.UUID("bc381540-e3e9-4026-b900-c2194bb6a17e")  # (read,notify)
_UART_RX_CHAR_UUID = bluetooth.UUID("39fb2309-6640-4dc5-a2cf-4b5c25b7ac35")  # (write)

class BLEuartCentral:
    def __init__(self, ble):
        self._ble = ble
        self._ble.active(True)     # bluetoothをアクティブにする
        self._ble.irq(self._irq)   # イベントのコールバック関数を登録する

        self._reset()              # 使用する変数の初期化

    def _reset(self):
        # スキャンが成功した時にキャッシュされる名前とアドレスの初期化。
        self._name = None
        self._addr_type = None
        self._addr = None

        # さまざまな操作を完了する為のコールバック。
        # これらは、呼び出された後、Noneにリセットされます。
        self._scan_callback = None
        self._conn_callback = None    # このサンプルはnoneのまま未使用
        self._read_callback = None    # このサンプルは未使用

        # 新しいデータがペリフェラルから通知された時の永続的なコールバック。
        self._notify_callback = None

        # 接続されたデバイス。
        self._conn_handle = None
        self._start_handle = None
        self._end_handle = None
        self._tx_handle = None
        self._rx_handle = None

    def _irq(self, event, data):
        if event == _IRQ_SCAN_RESULT:
            # シングルスキャン結果
            addr_type, addr, adv_type, rssi, adv_data = data
            _name = ''
            for i in _UART_DEVICE_NAME:
                _name += struct.pack("B", i).decode()
            if adv_type in (_ADV_IND, _ADV_DIRECT_IND) and _name in decode_name(adv_data):
                print('scan ok')
                # 潜在的なペリフェラルを見つけ、それを覚えてスキャンを停止します。
                self._addr_type = addr_type
                # addrバッファーは呼び出し元が所有している為、コピーする必要があります。
                self._addr = bytes(addr)
                self._name = _name or "?"
                self._ble.gap_scan(None)  # スキャンを停止

        elif event == _IRQ_SCAN_DONE:
            # スキャン期間が終了したか、手動で停止しました。
            if self._scan_callback:
                if self._addr:
                    # スキャン中にデバイスが見つかりました(スキャンが明示的に停止されました)。
                    self._scan_callback(self._addr_type, self._addr, self._name)
                    self._scan_callback = None
                else:
                    # スキャンがタイムアウトしました。
                    self._scan_callback(None, None, None)

        elif event == _IRQ_PERIPHERAL_CONNECT:
            # ペリフェラル(サーバー)に接続成功しました。
            conn_handle, addr_type, addr = data
            if addr_type == self._addr_type and addr == self._addr:
                self._conn_handle = conn_handle
                # 接続したサーバーに、サービスについて問い合わせます。
                self._ble.gattc_discover_services(self._conn_handle)

        elif event == _IRQ_PERIPHERAL_DISCONNECT:
            # 接続されているペリフェラルが切断されました。
            conn_handle, _, _ = data
            if conn_handle == self._conn_handle:
                self._reset()  # 使用する変数の初期化

        elif event == _IRQ_GATTC_SERVICE_RESULT:
            # 接続されたサーバーがサービスを返しました。
            conn_handle, start_handle, end_handle, uuid = data
            print("service", data)
            if conn_handle == self._conn_handle and uuid == _UART_SERVICE_UUID:
                self._start_handle, self._end_handle = start_handle, end_handle

        elif event == _IRQ_GATTC_SERVICE_DONE:
            # サービス検出が完了しました。
            print('_IRQ_GATTC_SERVICE_DONE')  # debug
            if self._start_handle and self._end_handle:
                # 接続したサーバーに、指定した範囲のキャラクタリスティックについて問い合わせます。
                self._ble.gattc_discover_characteristics(
                    self._conn_handle, self._start_handle, self._end_handle
                )
            else:
                # uartサービスが見つかりませんでした。
                print("Failed to find uart service.")

        elif event == _IRQ_GATTC_CHARACTERISTIC_RESULT:
            # 接続されたデバイスがuart特性(キャラクタリスティック)を返しました。
            conn_handle, def_handle, value_handle, properties, uuid = data
            if conn_handle == self._conn_handle and uuid == _UART_RX_CHAR_UUID:
                self._rx_handle = value_handle
            if conn_handle == self._conn_handle and uuid == _UART_TX_CHAR_UUID:
                self._tx_handle = value_handle

        elif event == _IRQ_GATTC_CHARACTERISTIC_DONE:
            # キャラクタリスティック検出が完了しました。
            print('_IRQ_GATTC_CHARACTERISTIC_DONE')  # debug
            if self._tx_handle is not None and self._rx_handle is not None:
                # サーバーの接続と検出が完了しました。接続コールバックを起動します。
                if self._conn_callback:
                    self._conn_callback()
            else:
                # UARTrx特性が見つかりませんでした。
                print("Failed to find uart rx characteristic.")

        elif event == _IRQ_GATTC_WRITE_DONE:
            # gattc_write()が完了しました。
            conn_handle, value_handle, status = data
            print("TX complete")

        elif event == _IRQ_GATTC_NOTIFY:
            # サーバーが通知要求を送信しました.
            print('_IRQ_GATTC_NOTIFY')  # debug
            conn_handle, value_handle, notify_data = data
            if conn_handle == self._conn_handle and value_handle == self._tx_handle:
                if self._notify_callback:
                    self._notify_callback(notify_data)  # 受信値の取り出し

    # 接続のチェックを行います、接続中の場合はtrueを返します。
    def is_connected(self):
        return (
            self._conn_handle is not None
            and self._tx_handle is not None
            and self._rx_handle is not None
        )

    # スキャンしてuartサービスをadvertisingするデバイスを見つけます。
    def scan(self, callback=None):
        self._addr_type = None
        self._addr = None
        self._scan_callback = callback
        # 3秒の間に50ms間隔で50ms期間スキャンを実行します。
        self._ble.gap_scan(3000, 50000, 50000)

    # 指定されたデバイスに接続します(それ以外の場合は、スキャンでキャッシュされたアドレスを使用します)。
    def connect(self, addr_type=None, addr=None, callback=None):
        self._addr_type = addr_type or self._addr_type
        self._addr = addr or self._addr
        self._conn_callback = callback
        if self._addr_type is None or self._addr is None:
            return False
        self._ble.gap_connect(self._addr_type, self._addr)  # ペリフェラルに接続します。
        return True

    # 現在のデバイスから切断します。(未使用)
    def disconnect(self):
        if not self._conn_handle:
            return
        self._ble.gap_disconnect(self._conn_handle)
        self._reset()  # 使用する変数の初期化

    # UARTを介してデータを送信します。
    def write(self, v, response=False):
        if not self.is_connected():
            return
        self._ble.gattc_write(self._conn_handle, self._rx_handle, v, 1 if response else 0)

    # デバイスが通知した時に呼び出されるコールバックを設定します。
    # UARTを介してデータを受信する時のハンドラーを設定します。
    def on_notify(self, callback):
        self._notify_callback = callback


def demo():
    ble = bluetooth.BLE()
    # ブルートゥースの有効
    central = BLEuartCentral(ble)

    not_found = False

    # デバイスが通知した時に呼び出されるコールバック関数
    def on_rx(v):
        _d = ''
        _i = ''
        _h = ''
        for x in v:
            _i += struct.pack("B",x).decode()
            _h += hex(x) + '/'
            _d += str(x)
        print('整数:{0} HEX:{1} 文字:{2}'.format(_d, _h, _i))

    # スキャン結果のコールバック関数
    def on_scan(addr_type, addr, name):
        if addr_type is not None:
            _rcv_adrs = ''
            for i in range(0, len(addr)):
                _rcv_adrs += hex(addr[i])[2:] + ':'
            print("Scan result found the peripheral:", addr_type, _rcv_adrs[:-1], name)
            central.connect()       # 指定されたデバイスに接続します
        else:
            nonlocal not_found      # デバイスが無い
            not_found = True
            print("No peripheral found.")

    central.scan(callback=on_scan)  # スキャンの実行

    # 接続を待つ...
    while not central.is_connected():
        time.sleep_ms(100)
        if not_found:               # デバイスが無いなら終了
            return

    print("Connected")

    central.on_notify(on_rx)        # デバイスから通知された時に呼び出されるコールバック関数の登録

    with_response = False

    i = 0
    while central.is_connected():
        try:
            v = str(i) + "_"
            print("TX", v)
            central.write(v, with_response)  # 応答無しのwrite書き込み
        except:
            print("TX failed")
        i += 1
        # ３秒毎の繰り返し
        time.sleep_ms(3000)

    print("Disconnected")


if __name__ == "__main__":
    demo()
